void WebHandleRoot() {
  server.send(200, "text/html",
              "<head><meta http-equiv=\"refresh\" content=\"" + String(SAMPLING_PERIODE_MS / 1000) + "\" ></head>"
              "<h1>You are connected to " + String(hostName) + "</h1>"
              "<fieldset>"
              "<legend>Sensors</legend>"
              "Temperature " + String(temp, 2) + "C<br/>"
              "Pressure " + String(pressure, 2) + "mB<br/>"
              "</fieldset>"
              "<fieldset>"
              "<legend>Pump</legend>"
              "<a href=\"/gpio?gpio=2&amp;value=1\">Power PUMP ON</a><br/>"
              "<a href=\"/gpio?gpio=2&value=0\">Power PUMP OFF</a><br/>"
              "</fieldset>"
              "<fieldset>"
              "<legend>Settings</legend>"
              "<a href=\"/setup\">Enter Setup</a><br/>"
              "<a href=\"/otamode\">Put device in OTA mode</a><br/>"
              "MQTT Status: " + (MqttIsConnected() ? "Connected" : "Disconnected") + "<br/>"
              "BMP 180 (Temp+Pression) Status: " + (BMP180IsConnected() ? "Connected" : "Disconnected") + "<br/>"
              "Free space: " + ESP.getFreeSketchSpace() + "<br/>"
              "Free heap: " + ESP.getFreeHeap() + "<br/>"
              "</fieldset>"
             );
}

void WebHandleSetup() {
  uint8_t mode;
  char *confSsid = "";
  char *confPassword = "";
  char *confHost = "";
  char *mqttServer = "";
  char *mqttUser = "";
  char *mqttPasswd = "";
  int mqttPort = 1883;

  EepromReadConfig(mode, &confSsid, &confPassword, &confHost, &mqttServer, &mqttUser, &mqttPasswd, mqttPort);

  server.send(200, "text/html", "<form action=\"/save\" method=\"get\">"
              "<fieldset>"
              "<legend>Wifi configuration:</legend>"
              "<div><label for=\"ssid\">Wifi SSID :</label> <br/><input type=\"text\" name=\"ssid\" value=\"" + String(confSsid) + "\" /></div>"
              "<div><label for=\"password\">Wifi Password :</label><br/><input type=\"password\" name=\"password\" /> </div>"
              "<div><label for=\"host\">Hostname :</label><br/><input type=\"text\" name=\"host\" value=\"" + String(confHost) + "\" /> </div>"
              "</fieldset>"
              "<fieldset>"
              "<legend>MQTT:</legend>"
              "<div><label for=\"mqttServer\">Server :</label><br/><input type=\"text\" name=\"mqttServer\" value=\"" + String(mqttServer) + "\" /> </div>"
              "<div><label for=\"mqttUser\">Username :</label><br/><input type=\"text\" name=\"mqttUser\" value=\"" + String(mqttUser) + "\" /> </div>"
              "<div><label for=\"mqttPasswd\">Password :</label><br/><input type=\"password\" name=\"mqttPasswd\" /> </div>"
              "<div><label for=\"mqttPort\">Port :</label><br/><input type=\"text\" name=\"mqttPort\" value=\"" + String(mqttPort) + "\" /> </div>"
              "</fieldset>"
              "<div class=\"button\"> <button type=\"submit\">Save</button></div>"
              "</form>");
}

void WebHandleGpio() {
  if (!server.hasArg("gpio") || !server.hasArg("value")) {
    server.send(500, "text/plain", "Bad arguments\r\n");
    return;
  }

  MqttChangeGpioValue(server.arg("gpio").toInt(), server.arg("value").toInt());
  server.send(200, "text/html", "<h1>GPIO" + server.arg("gpio") + " changed to " + server.arg("value") + "</h1>");
}

void WebHandleSave() {
  String password;
  String ssid;
  String hostName;
  String mqttServer;
  String mqttUser;
  String mqttPasswd;

  if (!server.hasArg("ssid") || !server.hasArg("password") || !server.hasArg("host") || !server.hasArg("mqttServer") || !server.hasArg("mqttUser") || !server.hasArg("mqttPasswd") || !server.hasArg("mqttPort") ) {
    server.send(500, "text/plain", "Bad arguments\r\n");
    return;
  }

  if (EepromSaveConfig(BOOTMODE_NORMAL, server.arg("ssid"), server.arg("password"), server.arg("host"), server.arg("mqttServer"), server.arg("mqttUser"), server.arg("mqttPasswd"), server.arg("mqttPort").toInt()) < 0) {
    server.send(500, "text/plain", "Cannot Save Credentials (Too long ?Contains \";\"?)\r\n");
    return;
  }

  server.send(200, "text/html", "<h1>Configuration Saved</h1><br/>"
              "<a href=\"/reboot\">Reboot</a><br/>");
}

void WebHandleOTA() {
  SKETCH_DEBUG_PRINTLN("Boot mode Set to OTA");
  EepromSaveBootMode(BOOTMODE_OTA);
  server.send(200, "text/html", "<h1>OTA Mode set</h1><br/>"
              "You can reboot now"
              "<a href=\"/reboot\">Reboot</a><br/>");
}

void WebHandleNotFound() {
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}

void WebHandleReboot() {
  SKETCH_DEBUG_PRINTLN("HTTP request to reboot");
  server.send(200, "text/html", "<h1>Device Reboot</h1><br/>");
  ESP.restart();
}

void WebSetupServer(int bootmode) {
  server.on("/", WebHandleRoot);
  server.on("/setup", WebHandleSetup);
  server.on("/save", WebHandleSave);
  server.on("/gpio", WebHandleGpio);
  server.on("/otamode", WebHandleOTA);
  server.on("/reboot", WebHandleReboot);

  server.onNotFound(WebHandleNotFound);
  server.begin();
  SKETCH_DEBUG_PRINTLN("HTTP server started");
}


